/**
 *      alphabeta.c
 *      Algorithme minmax optimisé alpha-beta
 *
 *      Mathieu Pellegrin
 *
 *      2009
 *
 **/

/* Inclusions standard */
#include <stdlib.h>
#include <stdio.h>

/* Inclusions du programme */
#include "../typedef.h"
#include "./../jeu/coups.h"
#include "./../jeu/damier.h"
#include "./../jeu/gestion_liste.h"
#include "evaluation.h"
#include "alphabeta.h"


/* Fonction d'exploration de l'arbre du minmax avec optimisation alpha-beta
 * Auteur: Mathieu Pellegrin
 * Entrée: Le coup de départ, le damier, le camp, la profondeur, et les paramètres alpha et beta
 * Sortie: L'évaluation du coup après parcours de l'arbre
 */
int alpha_beta(Tcoup cp, Tdamier dm, Tcouleur camp, int pr, int alpha, int beta)
{
	Tcoup *tmp = NULL;
	Tliste l = NULL;
	Tdamier dm2;
	int valeur = 0;
	int x,y;
	int coupealpha = 0;
	int coupebeta = 0;

	if (pr <= 1 || jeu_bloque(dm, camp))
	{
		valeur = evaluation(dm);
	}
	else
	{
		/* Copie du damier (oui c'est moche, lourd, mais c'est facile) */
		for (x=0; x<tailleDamier; x++)
		{
			for (y=0; y<tailleDamier; y++)
			{
				dm2[x][y].coul = dm[x][y].coul;
				dm2[x][y].piece = dm[x][y].piece;
			}
		}
		execute_coup(dm2, cp);
		generateur_coups(dm2, camp, &l);
		if (camp=='B')	/* Maximisation */
		{
			valeur = -MINMAX_INFINITE;
			tmp = l;
			while (!coupebeta && tmp!=NULL)
			{
				alpha = alpha_beta(*tmp, dm2, 'N', pr-1, alpha, beta);
				if (alpha >= beta)
					coupebeta = 1;
				tmp = tmp->suiv;
			}
			if (coupebeta)
				valeur = beta;
			else
				valeur = alpha;
			free_liste(&l);
			return valeur;
		}
		if (camp=='N') /* Minimisation */
		{
			valeur = MINMAX_INFINITE;
			tmp = l;
			while (!coupealpha && tmp!=NULL)
			{
				beta = alpha_beta(*tmp, dm2, 'B', pr-1, alpha, beta);
				if (alpha >= beta)
					coupealpha = 1;
				tmp = tmp->suiv;
			}
			if (coupealpha)
				valeur = alpha;
			else
				valeur = beta;
		}
		free_liste(&l);
	}
	return valeur;
}
