/**
 *      choix_coup.c
 *      Choix du meilleur coup
 *
 *      Mathieu Pellegrin
 *
 *      2009
 *
 **/

/* Inclusions standard */
#include <stdlib.h>
#include <stdio.h>

/* Inclusions du programme */
#include "../typedef.h"
#include "./../jeu/damier.h"
#include "minmax.h"
#include "alphabeta.h"
#include "choix_coup.h"


/* Choix du meilleur coup dans la liste l pour la couleur camp dans la position dm avec utilisation du minmax
 * Auteur: Mathieu Pellegrin
 * Entrée: Le camp, le damier (position), la liste de coups
 * Sortie: Le meilleur coup, valeur du meilleur coup
 */
Tcoup* choix_coup(Tcouleur camp, Tdamier dm, Tliste l, int *valeur)
{
	Tcoup* meilleur = l;
	int val;

	if (camp=='B')
	{
		/* Maximisation */
		*valeur = -1*MINMAX_INFINITE;
		while (l!=NULL)
		{
			val = alpha_beta(*l, dm, 'N', MINMAX_PROFMAX, -MINMAX_INFINITE, MINMAX_INFINITE);
			if (val > *valeur)
			{
				meilleur = l;
				*valeur = val;
			}
			l = l->suiv;
		}
	}
	if (camp=='N')
	{
		/* Minimisation */
		*valeur = MINMAX_INFINITE;
		while (l!=NULL)
		{
			val = alpha_beta(*l, dm, 'B', MINMAX_PROFMAX, -MINMAX_INFINITE, MINMAX_INFINITE);
			if (val < *valeur)
			{
				meilleur = l;
				*valeur = val;
			}
			l = l->suiv;
		}
	}
	return meilleur;
}
