/**
 *      minmax.c
 *      Algorithme minmax
 *
 *      Mathieu Pellegrin
 *
 *      2009
 *
 **/

/* Inclusions standard */
#include <stdlib.h>
#include <stdio.h>

/* Inclusions du programme */
#include "../typedef.h"
#include "./../jeu/coups.h"
#include "./../jeu/damier.h"
#include "./../jeu/gestion_liste.h"
#include "evaluation.h"
#include "minmax.h"


/* Fonction d'exploration de l'arbre du minmax
 * Auteur: Mathieu Pellegrin
 * Entrée: Le coup de départ, le damier, le camp, la profondeur
 * Sortie: L'évaluation du coup après parcours de l'arbre
 */
int min_max(Tcoup cp, Tdamier dm, Tcouleur camp, int pr)
{
	Tcoup *tmp = NULL;
	Tliste l = NULL;
	Tdamier dm2;
	int val, valeur = 0;
	int x,y;

	if (pr <= 1 || jeu_bloque(dm, camp))
	{
		valeur = evaluation(dm);
	}
	else
	{
		/* Copie du damier (oui c'est moche, lourd, mais c'est facile) */
		for (x=0; x<tailleDamier; x++)
		{
			for (y=0; y<tailleDamier; y++)
			{
				dm2[x][y].coul = dm[x][y].coul;
				dm2[x][y].piece = dm[x][y].piece;
			}
		}
		execute_coup(dm2, cp);
		generateur_coups(dm2, camp, &l);
		if (camp=='B')	/* Maximisation */
		{
			valeur = -MINMAX_INFINITE;
			tmp = l;
			while (tmp!=NULL)
			{
				val = min_max(*tmp, dm2, 'N', pr-1);
				if (val > valeur)
					valeur = val;
				tmp = tmp->suiv;
			}
			free_liste(&l);
			return valeur;
		}
		if (camp=='N') /* Minimisation */
		{
			valeur = MINMAX_INFINITE;
			tmp = l;
			while (tmp!=NULL)
			{
				val = min_max(*tmp, dm2, 'B', pr-1);
				if (val < valeur)
					valeur = val;
				tmp = tmp->suiv;
			}
		}
		free_liste(&l);
	}
	return valeur;
}
