/**
 *      interface_gtk.c
 *      Contient les fonctions de la gestion de l'interface en GTK (fonctions de callback principalement)
 *
 *      Mathieu Pellegrin
 *
 *      2009
 *
 **/

/* Inclusions standard */
#include <stdlib.h>
#include <stdio.h>
#include <gtk/gtk.h>

/* Inclusions du programme */
#include "../../typedef.h"
#include "gtkdef.h"
#include "../../common.h"
#include "../../jeu/coups.h"
#include "../../jeu/gestion_liste.h"
#include "../../jeu/damier.h"

#include "callback.h"
#include "interface_gtk.h"


/* Fonction de callback pour séléctionner une pièce
 * Auteur: Mathieu Pellegrin
 * Entrée: La structure de la partie
 * Sortie: (en paramètre) La structure de la partie
 */
void choisir_bouton(GtkWidget *widget, GdkEventButton *event, gpointer data)
{
	Tjeu jeu = (Tjeu)data;
	int stop = 0;
	int x,y;
	Tliste coup, tmp;
	char texte[50];
	GtkWidget *p_dialog = NULL;

	if ((jeu->tour=='B' && jeu->j_blanc == JOUEUR_IA) || ((jeu->tour=='N' && jeu->j_noir == JOUEUR_IA)))
	{
		p_dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, "C'est au tour de l'IA, cliquez sur Appliquer");
		gtk_dialog_run (GTK_DIALOG (p_dialog));
		gtk_widget_destroy (p_dialog);
	}
	else
	{
		/* Bon, c'est lequel celui là ? */
		for (x=0; x<tailleDamier && !stop; x++)
		{
			for (y=0; y<tailleDamier && !stop; y++)
			{
				if (jeu->cases[x][y]==widget)
					stop = 1;
			}
		}
		x--;
		y--;

		/* Chainage en fin de liste */
		tmp = jeu->coup;
		if (tmp!=NULL)
		{
			while (tmp->multiple!=NULL)
				tmp = tmp->multiple;
			if (tmp->arrivee.x < 0 || tmp->arrivee.y < 0)
			{
				tmp->arrivee.x = x;
				tmp->arrivee.y = y;
			}
			else
			{
				coup = (Tliste)malloc(sizeof(Tcoup));
				coup->multiple = NULL;
				coup->suiv = NULL;
				coup->depart.x = tmp->arrivee.x;
				coup->depart.y = tmp->arrivee.y;
				coup->arrivee.x = x;
				coup->arrivee.y = y;
				tmp->multiple = coup;
			}
		}
		else
		{
			coup = (Tliste)malloc(sizeof(Tcoup));
			coup->multiple = NULL;
			coup->suiv = NULL;
			coup->depart.x = x;
			coup->depart.y = y;
			coup->arrivee.x = -1;
			coup->arrivee.y = -1;
			jeu->coup = coup;
		}

		sprintf(texte,"%d ; %d",x+1,y+1);
		gtk_list_store_append(jeu->cases_selection, &(jeu->liste_iter));
		gtk_list_store_set(jeu->cases_selection, &(jeu->liste_iter), 0, texte, -1);
	}

	(void)event;
	/*(void)widget;*/
}


/* Fonction pour raffraichir l'affichage de la barre d'état
 * Auteur: Mathieu Pellegrin
 * Entrée: La structure de la partie
 * Sortie: (Ecran) La barre d'état
 */
void update_status(Tjeu jeu)
{
	char statusbar_text[100];
	sprintf(statusbar_text, STATUSBAR_FORMAT, jeu->tour, (jeu->j_blanc==JOUEUR_HUMAIN ? "Humain" : "IA"), (jeu->j_noir==JOUEUR_HUMAIN ? "Humain" : "IA"));
	gtk_statusbar_pop(GTK_STATUSBAR(jeu->statusbar), jeu->statusbar_context);
	gtk_statusbar_push(GTK_STATUSBAR(jeu->statusbar), jeu->statusbar_context,statusbar_text);
}


/* Fonction pour rafraichir l'affichage du damier
 * Auteur: Mathieu Pellegrin
 * Entrée: La structure de la partie
 * Sortie: (Ecran) Le damier
 */
void refresh_table(Tjeu jeu)
{
	int x,y;


	for (y=0; y<tailleDamier; y++)
	{
		for (x=0; x<tailleDamier; x++)
		{
			gtk_container_remove(GTK_CONTAINER(jeu->cases[x][tailleDamier-y-1]), GTK_WIDGET(jeu->labels[x][tailleDamier-y-1]));

			if (jeu->damier[x][tailleDamier-y-1].coul!='V')
			{
				if (jeu->damier[x][tailleDamier-y-1].piece=='P')
					if (jeu->damier[x][tailleDamier-y-1].coul=='B')
						jeu->labels[x][tailleDamier-y-1] = gtk_image_new_from_file(IMAGE_PION_BLANC);
					else
						jeu->labels[x][tailleDamier-y-1] = gtk_image_new_from_file(IMAGE_PION_NOIR);
				else
					if (jeu->damier[x][tailleDamier-y-1].coul=='B')
						jeu->labels[x][tailleDamier-y-1] = gtk_image_new_from_file(IMAGE_DAME_BLANC);
					else
						jeu->labels[x][tailleDamier-y-1] = gtk_image_new_from_file(IMAGE_DAME_NOIR);
			}
			else
			{
				if ((x+y+1)%2)
					jeu->labels[x][tailleDamier-y-1] = gtk_image_new_from_file(IMAGE_CASE_BLANC);
				else
					jeu->labels[x][tailleDamier-y-1] = gtk_image_new_from_file(IMAGE_CASE_NOIR);
				/*jeu->labels[x][tailleDamier-y-1] = gtk_image_new_from_file("interface/gtk/images/image32x32.png");*/
				/*jeu->labels[x][tailleDamier-y-1] = gtk_image_new_from_file(NULL);*/
			}

			gtk_container_add(GTK_CONTAINER (jeu->cases[x][tailleDamier-y-1]), jeu->labels[x][tailleDamier-y-1]);
			gtk_widget_show(jeu->labels[x][tailleDamier-y-1]);
		}
	}

	update_status(jeu);
}


/* Initialisation du tableau
 * Auteur: Mathieu Pellegrin
 * Entrée: La structure de la partie
 * Sortie: [Ecran] Le damier
 */
void init_tableau(GtkWidget* p_box, Tjeu jeu)
{
	GtkWidget *table;
	int x,y;
	char s[5];


	/*boutons = (GtkWidget***)malloc(sizeof(GtkWidget**)*tailleDamier*tailleDamier);*/

	table = gtk_table_new(tailleDamier+2,tailleDamier+2,TRUE);

	gtk_box_pack_start (GTK_BOX (p_box), table, FALSE, FALSE, 0);

	/*gtk_container_add(GTK_CONTAINER(p_box), GTK_WIDGET(table));*/

	for (x=0; x<tailleDamier; x++)
	{
		sprintf(s, "%i", x+1);
		gtk_table_attach(GTK_TABLE(table),
			gtk_label_new(s),
		 x+1, x+1+1, 0, 1, GTK_EXPAND , GTK_EXPAND, 0, 0);
		gtk_table_attach(GTK_TABLE(table),
			gtk_label_new(s),
		 x+1, x+1+1, tailleDamier+1, tailleDamier+1+1, GTK_EXPAND , GTK_EXPAND, 0, 0);
		sprintf(s, "%i", tailleDamier-x);
		gtk_table_attach(GTK_TABLE(table),
			gtk_label_new(s),
		 0, 1, x+1, x+1+1, GTK_EXPAND , GTK_EXPAND, 0, 0);
		gtk_table_attach(GTK_TABLE(table),
			gtk_label_new(s),
		 tailleDamier+1, tailleDamier+1+1, x+1, x+1+1, GTK_EXPAND , GTK_EXPAND, 0, 0);
	}

	for (y=0; y<tailleDamier; y++)
	{
		for (x=0; x<tailleDamier; x++)
		{
			jeu->cases[x][tailleDamier-y-1] = gtk_event_box_new();
			/*gtk_button_set_relief(GTK_BUTTON(jeu->cases[x][tailleDamier-y-1]), GTK_RELIEF_HALF);*/
			gtk_widget_add_events(jeu->cases[x][tailleDamier-y-1],GDK_BUTTON_PRESS_MASK);
			g_signal_connect(jeu->cases[x][tailleDamier-y-1], "button-press-event",G_CALLBACK(choisir_bouton), jeu);

			if (jeu->damier[x][tailleDamier-y-1].coul!='V')
			{
				if (jeu->damier[x][tailleDamier-y-1].piece=='P')
					if (jeu->damier[x][tailleDamier-y-1].coul=='B')
						jeu->labels[x][tailleDamier-y-1] = gtk_image_new_from_file(IMAGE_PION_BLANC);
					else
						jeu->labels[x][tailleDamier-y-1] = gtk_image_new_from_file(IMAGE_PION_NOIR);
				else
					if (jeu->damier[x][tailleDamier-y-1].coul=='B')
						jeu->labels[x][tailleDamier-y-1] = gtk_image_new_from_file(IMAGE_DAME_BLANC);
					else
						jeu->labels[x][tailleDamier-y-1] = gtk_image_new_from_file(IMAGE_DAME_NOIR);
			}
			else
			{
				if ((x+y+1)%2)
					jeu->labels[x][tailleDamier-y-1] = gtk_image_new_from_file(IMAGE_CASE_BLANC);
				else
					jeu->labels[x][tailleDamier-y-1] = gtk_image_new_from_file(IMAGE_CASE_NOIR);
				/*jeu->labels[x][tailleDamier-y-1] = gtk_image_new_from_file("interface/gtk/images/image32x32.png");*/
				/*jeu->labels[x][tailleDamier-y-1] = gtk_image_new_from_file(NULL);*/
			}

			gtk_container_add(GTK_CONTAINER (jeu->cases[x][tailleDamier-y-1]), jeu->labels[x][tailleDamier-y-1]);
			gtk_table_attach(GTK_TABLE(table), jeu->cases[x][tailleDamier-y-1], x+1, x+1+1, y+1, y+1+1, GTK_EXPAND , GTK_EXPAND, 0, 0);
		}
	}

	return;
}


/* Choix des blancs
 * Auteur: Mathieu Pellegrin
 * Entrée: La structure de la partie
 * Sortie: (paramètre) La structure de la partie
 */
void choix_blanc(GtkWidget *widget, gpointer user_data)
{
	Tjeu jeu = (Tjeu)user_data;

	jeu->j_blanc = JOUEUR_HUMAIN;
	jeu->j_noir = JOUEUR_IA;

	update_status(jeu);

	(void)widget;
}

/* Choix des noirs
 * Auteur: Mathieu Pellegrin
 * Entrée: La structure de la partie
 * Sortie: (paramètre) La structure de la partie
 */
void choix_noir(GtkWidget *widget, gpointer user_data)
{
	Tjeu jeu = (Tjeu)user_data;

	jeu->j_noir = JOUEUR_HUMAIN;
	jeu->j_blanc = JOUEUR_IA;

	update_status(jeu);

	(void)widget;
}

/* Demande du camp du joueur contre l'IA
 * Auteur: Mathieu Pellegrin
 * Entrée: La structure de la partie
 * Sortie: (paramètre) La structure de la partie
 */
void demander_camp(Tjeu jeu)
{
	GtkWidget *dialog, *label;
	GtkWidget *blanc, *noir;

	choix_blanc(NULL,jeu);
	blanc = gtk_radio_button_new_with_label(NULL, "Les  blancs");
	noir = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON (blanc), "Les noirs");

	/* Créer les widgets */
	dialog = gtk_dialog_new_with_buttons ("Choix du camp", NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK, GTK_RESPONSE_NONE, NULL);
	label = gtk_label_new ("Le joueur humain joue avec :");

	/* S'assurer que la boîte de dialogue est détruite quand l'utilisateur a répondu. */
	g_signal_connect_swapped(dialog, "response", G_CALLBACK (gtk_widget_destroy), dialog);
	g_signal_connect(blanc, "clicked", G_CALLBACK (choix_blanc), jeu);
	g_signal_connect(noir, "clicked", G_CALLBACK (choix_noir), jeu);

	/*Ajouter le label, et afficher tout ce qui a été ajouté à la boîte de dialogue. */
	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), label);
	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), blanc);
	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), noir);
	gtk_widget_show_all (dialog);
}

