/**
 *      menu.c
 *      Contient les fonctions pour créer le menu
 *
 *      Mathieu Pellegrin
 *
 *      2009
 *
 **/

/* Inclusions standard */
#include <stdlib.h>
#include <stdio.h>
#include <gtk/gtk.h>

/* Inclusions du programme */
#include "../../typedef.h"
#include "../../common.h"

#include "../common_interface.h"
#include "gtkdef.h"
/*#include "interface_gtk.h"*/
#include "partie_gtk.h"
#include "menu.h"
#include "callback.h"
#include "help_about.h"

static void cb_menu_addWidget (GtkUIManager *, GtkWidget *, GtkContainer *);


/* Initialisation du menu
 * Auteur: Mathieu Pellegrin
 * Entrée: La structure de la partie
 * Sortie: (paramètre) La structure de la partie
 */
void init_menu(GtkWidget* p_box, Tjeu jeu)
{
	GtkUIManager *p_uiManager = NULL;
	GtkActionGroup *p_actionGroup = NULL;

	GtkActionEntry entries[] = {
		{"FichierMenuAction", NULL, "_Partie", NULL, NULL, NULL},
		{"NouveauAction", GTK_STOCK_NEW, "_Nouveau", NULL, NULL, NULL },
		{"ReloadAction", GTK_STOCK_REVERT_TO_SAVED, "Recharger", "<Control>R", "Recommencer la partie en cour", G_CALLBACK (reinit_table)},
		{"IAAction", GTK_STOCK_PROPERTIES, "Jeu IA", "", "Demander à l'IA de jouer ce coup", G_CALLBACK (ia_joue)},
		{"PasserAction", GTK_STOCK_REDO, "Passer", "", "Passer son tour", G_CALLBACK (passer_tour)},
		{"PartieHvsH", GTK_STOCK_NEW, "Partie Humain-Humain", "<Control>1", NULL, G_CALLBACK (start_new_humain_vs_humain)},
		{"PartieHvsI", GTK_STOCK_NEW, "Partie Humain-IA", "<Control>2", NULL, G_CALLBACK (start_new_humain_vs_ia)},
		{"PartieIvsI", GTK_STOCK_NEW, "Partie IA-IA", "<Control>3", NULL, G_CALLBACK (start_new_ia_vs_ia)},
		{"OuvrirAction", GTK_STOCK_OPEN, "_Charger...", "<Control>O", "Charger une partie", G_CALLBACK (charge_partie)},
		{"SauverAction", GTK_STOCK_SAVE, "_Sauver...", "<Control>S", NULL, G_CALLBACK (sauve_partie)},
		{"QuitterAction", GTK_STOCK_QUIT, "_Quitter", "<control>Q", NULL, G_CALLBACK (cb_quit)},
		{"AideMenuAction", NULL, "_Aide", NULL, NULL, NULL},
		{"AideAction", GTK_STOCK_HELP, "_Aide", "<Release>F1", NULL, G_CALLBACK (cb_menu_help)},
		{"AproposAction", GTK_STOCK_ABOUT, "_A propos", "<Control>A", NULL, G_CALLBACK (cb_menu_about)}
	};

	p_uiManager = gtk_ui_manager_new();

	gtk_ui_manager_add_ui_from_file(p_uiManager,"interface/gtk/menu.xml", NULL);

	p_actionGroup = gtk_action_group_new( "menuActionGroup" );

	gtk_action_group_add_actions( p_actionGroup, entries, G_N_ELEMENTS( entries ), (gpointer)jeu );

	gtk_ui_manager_insert_action_group( p_uiManager, p_actionGroup, 0 );

	g_signal_connect(p_uiManager, "add_widget", G_CALLBACK(cb_menu_addWidget), p_box);

	gtk_ui_manager_ensure_update(p_uiManager);

}

static void cb_menu_addWidget (GtkUIManager * p_uiManager, GtkWidget * p_widget, GtkContainer * p_box)
{
	gtk_box_pack_start (GTK_BOX (p_box), p_widget, FALSE, FALSE, 0);
	gtk_widget_show (p_widget);

	(void) p_uiManager;

	return;
}

