/**
 *      interface_texte.c
 *      Contient les fonctions de la gestion de l'interface en mode texte
 *
 *      Mathieu Pellegrin
 *
 *      2009
 *
 **/

/* Inclusions standard */
#include <stdlib.h>
#include <stdio.h>

/* Inclusions du programme */
#include "../../typedef.h"
#include "../../common.h"
#include "../../jeu/damier.h"
#include "../../jeu/pieces.h"
#include "../../jeu/coups.h"
#include "../common_interface.h"
#include "colors.h"
#include "interface_texte.h"
#include "terminal_hacks.h"

/* Initialise le terminal
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier
 * Sortie: (écran) le damier
 */
void init_interface_texte()
{

}


/* Affichage du damier
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier
 * Sortie: (écran) le damier
 */
void affiche_damier(Tdamier dm)
{
	Tcoup cp;
	cp.depart.x = -1;
	cp.depart.y = -1;
	cp.arrivee.x = -1;
	cp.suiv = NULL;
	cp.multiple = NULL;
	affiche_damier_curseur(dm,cp);
	return;
/*
	int x,y;
	printf("    ");
	for (x=0; x<tailleDamier; x++)
		printf("%d ",x);
	printf("\n");
	printf("  - ");
	for (x=0; x<tailleDamier; x++)
		printf("- ");
	printf("-   ");
	printf("\n");
	for (y=0; y<tailleDamier; y++)
	{
		printf("%d | ",tailleDamier-1-y);
		for (x=0; x<tailleDamier; x++)
		{
			if (dm[x][tailleDamier-y-1].coul=='V')
				printf(". ");
			else
				if (dm[x][tailleDamier-y-1].piece=='P')
					printf("%c ",(dm[x][tailleDamier-y-1].coul=='B' ? 'b' : 'n'));
				else
					printf("%c ",dm[x][tailleDamier-y-1].coul);
		}
		printf("| %d\n",tailleDamier-1-y);
	}
	printf("  - ");
	for (x=0; x<tailleDamier; x++)
		printf("- ");
	printf("-   ");
	printf("\n");
	printf("    ");
	for (x=0; x<tailleDamier; x++)
		printf("%d ",x);
	printf("\n");
*/
}


/* Affiche le damier avec curseur
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier
 * Sortie: (écran) le damier
 */
void affiche_damier_curseur(Tdamier dm, Tcoup cp)
{
	int x,y;
	Tliste tmp, last_cp;
	int highlight, current;

	last_cp = &cp;
	while (last_cp->multiple!=NULL)
		last_cp = last_cp->multiple;

	printf("    %c%s  %c%s",ESC,BORDER_TEXT_COLOR,ESC,END_BORDER_TEXT_COLOR);
	for (x=0; x<tailleDamier; x++)
		printf("%c%s%d  %c%s",ESC,BORDER_TEXT_COLOR,x,ESC,END_BORDER_TEXT_COLOR);
	printf("\n");

	printf("  %c%s -  %c%s",ESC,BORDER_COLOR,ESC,END_BORDER_COLOR);
	printf("%c%s- %c%s",ESC,BORDER_COLOR,ESC,END_BORDER_COLOR);
	for (x=0; x<tailleDamier; x++)
		printf("%c%s - %c%s",ESC,BORDER_COLOR,ESC,END_BORDER_COLOR);
	printf("\n");

	for (y=0; y<tailleDamier; y++)
	{
		printf(" %c%s%d%c%s%c%s | %c%s",ESC,BORDER_TEXT_COLOR,tailleDamier-1-y,ESC,END_BORDER_TEXT_COLOR,ESC,BORDER_COLOR,ESC,END_BORDER_COLOR);
		for (x=0; x<tailleDamier; x++)
		{

			tmp = &cp;
			while (tmp!=NULL && !(x==tmp->depart.x && tailleDamier-y-1==tmp->depart.y))
				tmp = tmp->multiple;
			if (tmp!=NULL)
				highlight = 1;
			else
				highlight = 0;

			if (last_cp->arrivee.x==x && last_cp->arrivee.y==tailleDamier-y-1)
				current = 1;
			else
				current = 0;

			if ((x+y+1)%2)
				printf("%c%s",ESC,BLACK_SQUARES_COLOR);
			if (current)
				printf("%c[5m[%c[25m%c[1m",0x1B,0x1B,0x1B);
			else
			{
				if (highlight)
					printf("%c[7m*",0x1B);
				else
					printf(" ");
			}
			if (dm[x][tailleDamier-y-1].coul=='V')
				printf(" ");
			else
				if (dm[x][tailleDamier-y-1].piece=='P')
					printf("%c",(dm[x][tailleDamier-y-1].coul=='B' ? 'b' : 'n'));
				else
					printf("%c",dm[x][tailleDamier-y-1].coul);
			if (current)
				printf("%c[22m%c[5m]%c[25m",0x1B,0x1B,0x1B);
			else
				if (highlight)
					printf("*%c[0m",0x1B);
				else
					printf(" ");
			if ((x+y+1)%2)
				printf("%c%s",ESC,END_BLACK_SQUARES_COLOR);
		}
		printf("%c%s | %c%s%c%s%d%c%s\n",ESC,BORDER_COLOR,ESC,END_BORDER_COLOR,ESC,BORDER_TEXT_COLOR,tailleDamier-1-y,ESC,END_BORDER_TEXT_COLOR);
	}
	printf("  %c%s -  %c%s",ESC,BORDER_COLOR,ESC,END_BORDER_COLOR);
	printf("%c%s- %c%s",ESC,BORDER_COLOR,ESC,END_BORDER_COLOR);
	for (x=0; x<tailleDamier; x++)
		printf("%c%s - %c%s",ESC,BORDER_COLOR,ESC,END_BORDER_COLOR);
	printf("\n");
	printf("    %c%s  %c%s",ESC,BORDER_TEXT_COLOR,ESC,END_BORDER_TEXT_COLOR);
	for (x=0; x<tailleDamier; x++)
		printf("%c%s%d  %c%s",ESC,BORDER_TEXT_COLOR,x,ESC,END_BORDER_TEXT_COLOR);
	printf("\n");
}


/* Saisie d'un coup au clavier en saisissant les coordonnées
 * Auteur: Mathieu Pellegrin
 * Entrée: Le coup, et la liste de coups
 * Sortie: affecte *cp avec un coup saisi au clavier
 */
void saisie_clavier_coup_coord(Tcoup *cp)
{
	do
	{
		viderTamponEntree();
		printf("Positions de départ ? (x y) ");
	} while (scanf("%d %d", &((*cp).depart.x), &((*cp).depart.y))<2);
	do
	{
		viderTamponEntree();
		printf("Positions d'arrivée ? (x y) ");
	} while (scanf("%d %d", &((*cp).arrivee.x), &((*cp).arrivee.y))<2);
}


/* Saisie d'un coup au clavier en utilisant un curseur
 * Auteur: Mathieu Pellegrin
 * Entrée: Le coup, et la liste de coups
 * Sortie: affecte *cp avec un coup saisi au clavier
 */
int saisie_clavier_coup_curseur(Tdamier dm, Tcoup *cp)
{
	char c;
	int depart = 1;
	int stop = 0;
	int quit = 0;
	int valide = 0;
	int refresh = 1;
	Tliste first_cp = NULL;
	Tliste prec = NULL;
	Tliste tmp = NULL;
	Tliste *last_cp = NULL;

	tmp = (Tliste)malloc(sizeof(Tcoup));

	first_cp = tmp;
	prec = tmp;
	last_cp  = &(tmp->multiple);

	tmp->depart.x = tailleDamier/2;
	tmp->depart.y = tailleDamier/2;
	tmp->arrivee.x = tailleDamier/2;
	tmp->arrivee.y = tailleDamier/2;
	tmp->suiv = NULL;
	tmp->multiple = NULL;

	do
	{
		if (refresh)
		{
			if (depart)
				printf("\nSaisir la position de départ\n");
			else
				printf("\nSaisir la position d'arrivée\n");
			printf("Entrée ou 'F' pour valider\n");
			printf("'Q' pour quitter\n\n");

				affiche_damier_curseur(dm, *first_cp);
		}

		/* Set raw mode on stdin. */
		if(ttyraw(0) < 0)
		{
			fprintf(stderr,"Can't go to raw mode.\n");
			exit(1);
		}

		c = getchar();

		/* Unset raw mode on stdin. */
		if(ttyreset(0) < 0)
		{
			fprintf(stderr, "Cannot reset terminal!\n");
			exit(-1);
		}

		switch (c)
		{
			case 'A':
				if (depart)
					tmp->depart.y = (tmp->depart.y<tailleDamier-1 ? tmp->depart.y+1 : tmp->depart.y);
				else
					tmp->arrivee.y = (tmp->arrivee.y<tailleDamier-1 ? tmp->arrivee.y+1 : tmp->arrivee.y);
			break;

			case 'B':
				if (depart)
					tmp->depart.y = (tmp->depart.y>0 ? tmp->depart.y-1 : tmp->depart.y);
				else
					tmp->arrivee.y = (tmp->arrivee.y>0 ? tmp->arrivee.y-1 : tmp->arrivee.y);
			break;

			case 'C':
				if (depart)
					tmp->depart.x = (tmp->depart.x<tailleDamier-1 ? tmp->depart.x+1 : tmp->depart.x);
				else
					tmp->arrivee.x = (tmp->arrivee.x<tailleDamier-1 ? tmp->arrivee.x+1 : tmp->arrivee.x);
			break;

			case 'D':
				if (depart)
					tmp->depart.x = (tmp->depart.x>0 ? tmp->depart.x-1 : tmp->depart.x);
				else
					tmp->arrivee.x = (tmp->arrivee.x>0 ? tmp->arrivee.x-1 : tmp->arrivee.x);
			break;

			case 13:
			case 'f':
			case 'F':
				valide = 1;
			break;

			case 'q':
			case 'Q':
				quit = 1;
				stop = 1;
			break;

			default:
				refresh = 0;
				/*printf("%d",c);*/
			break;

		}

		if (depart && valide)
		{
			depart = 0;
			valide = 0;
		}
		else
		{
			if (depart && !valide)
			{
				tmp->arrivee.x = tmp->depart.x;
				tmp->arrivee.y = tmp->depart.y;
			}
			else
			{
				if (valide && !depart)
				{
					valide = 0;
/*
					if (tmp!=first_cp)
					{
						tmp->depart.x = prec->arrivee.x;
						tmp->depart.y = prec->arrivee.y;
					}
*/
					*last_cp = tmp;
					last_cp = &(tmp->multiple);

					if (prec!=tmp && prec->arrivee.x==tmp->arrivee.x && prec->arrivee.y==tmp->arrivee.y)
					{
						free(tmp);
						prec->multiple = NULL;
						stop = 1;
					}
					else
					{
						prec = tmp;
						tmp = (Tliste)malloc(sizeof(Tcoup));

						last_cp = &(tmp->multiple);
						prec->multiple = tmp;

						tmp->depart.x = prec->arrivee.x;
						tmp->depart.y = prec->arrivee.y;
						tmp->arrivee.x = prec->arrivee.x;
						tmp->arrivee.y = prec->arrivee.y;
						tmp->suiv = NULL;
						tmp->multiple = NULL;
					}
				}
			}
		}

		/* Efface l'écran */
		if (refresh)
			system("clear");

		refresh = 1;

	} while (!stop);

	cp->depart.x = first_cp->depart.x;
	cp->depart.y = first_cp->depart.y;
	cp->arrivee.x = first_cp->arrivee.x;
	cp->arrivee.y = first_cp->arrivee.y;
	cp->suiv = first_cp->suiv;
	cp->multiple = first_cp->multiple;

	free(first_cp);

	return quit;
}


/* Lecture de la liste de coup ("jeu de la partie")
 * Auteur: Mathieu Pellegrin
 * Entrée: La liste de coups, le damier
 * Sortie: [Ecran]
 */
void lit_sauvegarde(Tliste liste, Tdamier damier)
{
	Tcoup coup;
	init_damier(damier);
	affiche_damier(damier);
	getchar();

	while(liste!=NULL)
	{
		coup.depart.x = (liste->depart).x;
		coup.depart.y = (liste->depart).y;
		coup.arrivee.x = (liste->arrivee).x;
		coup.arrivee.y = (liste->arrivee).y;
		coup.multiple = liste->multiple;
		getchar();
		execute_coup(damier, coup);
		affiche_damier(damier);
		liste = liste->suiv;
	}
}
