/**
 *      partie_texte.c
 *      Contient les fonctions pour la gestion du jeu en mode texte
 *
 *      Mathieu Pellegrin
 *
 *      2009
 *
 **/

/* Inclusions standard */
#include <stdlib.h>
#include <stdio.h>

/* Inclusions du programme */
#include "../../typedef.h"
#include "../../common.h"
#include "../../jeu/gestion_liste.h"
#include "../../jeu/gestion_sauvegarde.h"
#include "../../jeu/damier.h"
#include "../../jeu/pieces.h"
#include "../../jeu/coups.h"
#include "../../jeu/jeu.h"
#include "../../ia/choix_coup.h"
#include "interface_texte.h"
#include "partie_texte.h"


/* Procédure permettant la gestion d'une partie opposant deux joueurs humains sur la base d'une interface limitée à une saisie clavier des coups et un affichage en mode texte du damier
 * Auteur: Mathieu Pellegrin
 * Entrée: Rien
 * Sortie: [écran]
 */
void jeu_humain_vs_humain(Tliste listecoups)
{
	Tcouleur joueur ='B';
	Tcouleur coul = 0;
	Tcoup coup;
	Tdamier damier;
	Tliste liste;
	int stop = 0;

	init_damier(damier);
	init_liste(&liste);

	lit_sauvegarde_unlock(listecoups, damier);
	inverse_liste(&listecoups);	/* Oui, la sauvegarde est à l'envers parce que c'est plus facile... */

	do
	{
		generateur_coups(damier, joueur, &liste);

		/* --- DEBUG --- */
		#if DEBUG_LVL>=1
		affiche_liste(liste);
		#endif
		/* --- /DEBUG --- */

		do
		{
			system("clear");
			printf("\nJoueur %c \n",joueur);
			/*affiche_damier(damier);

			saisie_clavier_coup_coord(&coup);*/
			stop = saisie_clavier_coup_curseur(damier,&coup);

			if (limites_damier(coup.depart) && limites_damier(coup.arrivee))
				coul = damier[coup.depart.x][coup.depart.y].coul;

		} while(!stop && liste!=NULL && (coul!=joueur || !verification_coup(coup, liste)));

		if (!stop)
		{
			execute_coup(damier, coup);
			sauve_coup(&listecoups, coup);
			sauve_liste_coup(listecoups, FICHIER_SAUVEGARDE);
			if (coup.multiple != NULL)
			{
				free_maillon(coup.multiple);
				coup.multiple = NULL;
			}
			inverser_couleur(&joueur);
		}
		free_liste(&liste);

	} while(!stop && !fin_partie(damier,coup));

	if (!stop)
		sauve_liste_coup(listecoups, FICHIER_SAUVEGARDE);
	free_liste(&listecoups);
}


/* Procédure permettant la gestion d'une partie opposant un joueurs humains à l'IA basée sur le minmax, sur la base d'une interface limitée à une saisie clavier des coups et un affichage en mode texte du damier
 * Auteur: Mathieu Pellegrin
 * Entrée: Rien
 * Sortie: [écran]
 */
void jeu_humain_vs_ia(Tliste listecoups)
{
	Tcouleur joueur ='B';
	Tcouleur coul = 0;
	Tcoup coup;
	Tcoup *coup_ia;
	Tdamier damier;
	Tliste liste;
	int valeur;
	int stop = 0;

	init_damier(damier);
	init_liste(&liste);

	lit_sauvegarde_unlock(listecoups, damier);
	inverse_liste(&listecoups);	/* Oui, la sauvegarde est à l'envers parce que c'est plus facile... */

	coup.multiple = NULL;

	do
	{
		generateur_coups(damier, joueur, &liste);

		/* --- DEBUG --- */
		#if DEBUG_LVL>=1
		affiche_liste(liste);
		#endif
		/* --- /DEBUG --- */

		do
		{
			system("clear");
			printf("\nJoueur %c \n",joueur);

			if (joueur=='B')
				stop = saisie_clavier_coup_curseur(damier,&coup);
			else
			{
				coup_ia = choix_coup(joueur, damier, liste, &valeur);
				if (coup_ia != NULL)
					coup = *coup_ia;
				else
					stop = 1;
			}

			if (limites_damier(coup.depart) && limites_damier(coup.arrivee))
				coul = damier[coup.depart.x][coup.depart.y].coul;

		} while(!stop && liste!=NULL && (coul!=joueur || !verification_coup(coup, liste)));

		if (!stop)
		{
			execute_coup(damier, coup);
			sauve_coup(&listecoups, coup);
			sauve_liste_coup(listecoups, FICHIER_SAUVEGARDE);
			if (joueur=='B' && coup.multiple != NULL)
			{
				free_maillon(coup.multiple);
				coup.multiple = NULL;
			}
			inverser_couleur(&joueur);
		}
		free_liste(&liste);


	} while(!stop && !fin_partie(damier,coup));

	if (!stop)
		sauve_liste_coup(listecoups, FICHIER_SAUVEGARDE);
	free_liste(&listecoups);
}


/* Procédure permettant la gestion d'une partie opposant deux joueurs IA basée sur le minmax, sur la base d'une interface limitée à une saisie clavier des coups et un affichage en mode texte du damier
 * Auteur: Mathieu Pellegrin
 * Entrée: Rien
 * Sortie: [écran]
 */
void jeu_ia_vs_ia(Tliste listecoups)
{
	Tcouleur joueur ='B';
	Tcouleur coul = 0;
	Tcoup coup;
	Tcoup *coup_ia;
	Tdamier damier;
	Tliste liste;
	int valeur;
	int stop = 0;

	init_damier(damier);
	init_liste(&liste);

	lit_sauvegarde_unlock(listecoups, damier);
	inverse_liste(&listecoups);	/* Oui, la sauvegarde est à l'envers parce que c'est plus facile... */

	do
	{
		generateur_coups(damier, joueur, &liste);

		do
		{
			system("clear");
			printf("\nJoueur %c \n",joueur);

			coup_ia = choix_coup(joueur, damier, liste, &valeur);
				if (coup_ia != NULL)
					coup = *coup_ia;
				else
					stop = 1;

			if (!stop && limites_damier(coup.depart) && limites_damier(coup.arrivee))
				coul = damier[coup.depart.x][coup.depart.y].coul;

		} while(!stop && liste!=NULL && (coul!=joueur || !verification_coup(coup, liste)));

		/* --- DEBUG --- */
		#if DEBUG_LVL>=1
		affiche_damier(damier);
		affiche_liste(liste);
		printf("%d %d %d %d\n",coup.depart.x,coup.depart.y,coup.arrivee.x,coup.arrivee.y);
		getchar();
		#endif
		/* --- /DEBUG --- */

		if (!stop)
		{
			execute_coup(damier, coup);
			inverser_couleur(&joueur);
			sauve_coup(&listecoups, coup);
		}
		free_liste(&liste);

	} while(!stop && !fin_partie(damier,coup));

	if (!stop)
		sauve_liste_coup(listecoups, FICHIER_SAUVEGARDE);
	free_liste(&listecoups);
}

