/**
 *      damier.c
 *      Contient les fonctions de la gestion des coups
 *
 *      Mathieu Pellegrin
 *
 *      2009
 *
 **/

/* Inclusions standard */
#include <stdlib.h>
#include <stdio.h>

/* Inclusions du programme */
#include "../typedef.h"
#include "../common.h"
#include "pieces.h"
#include "gestion_liste.h"
#include "prises_multiples.h"
#include "damier.h"
#include "coups.h"


/* Remplit la liste des coups possibles pour un pion
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier, la liste chainée, les coordonnés du pion, si le paramètre prise vaut 1, seules les prises seront considérées
 * Sortie: (paramètre) construit la liste associée à *l et qui contient tous les coups qui sont possibles, prise et déplacement pour le pion considérée
 */
void coups_possibles_simples_pion(Tdamier dm, Tcoord coord, Tliste *l, int *prise)
{
	Tcoup tmp;

	/* Prises */
	if (dm[coord.x][coord.y].piece == 'P')
	{
		if (ppavg(dm,coord))
		{
			*prise = 1;
			tmp.depart.x = coord.x;
			tmp.depart.y = coord.y;
			tmp.arrivee.x = coord.x-2;
			tmp.arrivee.y = coord.y+2;
			tmp.suiv = NULL;
			tmp.multiple = NULL;
			ajoute_element(l,tmp);
		}
		if (ppavd(dm,coord))
		{
			*prise = 1;
			tmp.depart.x = coord.x;
			tmp.depart.y = coord.y;
			tmp.arrivee.x = coord.x+2;
			tmp.arrivee.y = coord.y+2;
			tmp.suiv = NULL;
			tmp.multiple = NULL;

			ajoute_element(l,tmp);
		}
		if (pparg(dm,coord))
		{
			*prise = 1;
			tmp.depart.x = coord.x;
			tmp.depart.y = coord.y;
			tmp.arrivee.x = coord.x-2;
			tmp.arrivee.y = coord.y-2;
			tmp.suiv = NULL;
			tmp.multiple = NULL;
			ajoute_element(l,tmp);
		}
		if (ppard(dm,coord))
		{
			*prise = 1;
			tmp.depart.x = coord.x;
			tmp.depart.y = coord.y;
			tmp.arrivee.x = coord.x+2;
			tmp.arrivee.y = coord.y-2;
			tmp.suiv = NULL;
			tmp.multiple = NULL;
			ajoute_element(l,tmp);
		}
	}

	/* Déplacements */
	if (!(*prise))
		{
		if (dm[coord.x][coord.y].piece == 'P')
		{
			if (pdavg(dm,coord))
			{
				tmp.depart.x = coord.x;
				tmp.depart.y = coord.y;
				tmp.arrivee.x = coord.x-1;
				tmp.arrivee.y = coord.y+1;
				tmp.suiv = NULL;
				tmp.multiple = NULL;
				ajoute_element(l,tmp);
			}
			if (pdavd(dm,coord))
			{
				tmp.depart.x = coord.x;
				tmp.depart.y = coord.y;
				tmp.arrivee.x = coord.x+1;
				tmp.arrivee.y = coord.y+1;
				tmp.suiv = NULL;
				tmp.multiple = NULL;
				ajoute_element(l,tmp);
			}
			if (pdarg(dm,coord))
			{
				tmp.depart.x = coord.x;
				tmp.depart.y = coord.y;
				tmp.arrivee.x = coord.x-1;
				tmp.arrivee.y = coord.y-1;
				tmp.suiv = NULL;
				tmp.multiple = NULL;
				ajoute_element(l,tmp);
			}
			if (pdard(dm,coord))
			{
				tmp.depart.x = coord.x;
				tmp.depart.y = coord.y;
				tmp.arrivee.x = coord.x+1;
				tmp.arrivee.y = coord.y-1;
				tmp.suiv = NULL;
				tmp.multiple = NULL;
				ajoute_element(l,tmp);
			}
		}
	}
}


/* Remplit la liste des coups possibles pour une dame
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier, la liste chainée, les coordonnés de la dame, si le paramètre prise vaut 1, seules les prises seront considérées
 * Sortie: (paramètre) construit la liste associée à *l et qui contient tous les coups qui sont possibles, prise et déplacement pour la dame considérée
 */
void coups_possibles_simples_dame(Tdamier dm, Tcoord coord, Tliste *l, int *prise)
{
	Tcoup tmp;
	int n;

	/* Prises */
	if (dm[coord.x][coord.y].piece == 'D')
	{
		for (n = 1; n<tailleDamier; n++)
		{
			if (dpavgn(dm,coord,n))
			{
				*prise = 1;
				tmp.depart.x = coord.x;
				tmp.depart.y = coord.y;
				tmp.arrivee.x = coord.x-n;
				tmp.arrivee.y = coord.y+n;
				tmp.suiv = NULL;
				tmp.multiple = NULL;
				ajoute_element(l,tmp);
			}
			if (dpavdn(dm,coord,n))
			{
				*prise = 1;
				tmp.depart.x = coord.x;
				tmp.depart.y = coord.y;
				tmp.arrivee.x = coord.x+n;
				tmp.arrivee.y = coord.y+n;
				tmp.suiv = NULL;
				tmp.multiple = NULL;
				ajoute_element(l,tmp);
			}
			if (dpargn(dm,coord,n))
			{
				*prise = 1;
				tmp.depart.x = coord.x;
				tmp.depart.y = coord.y;
				tmp.arrivee.x = coord.x-n;
				tmp.arrivee.y = coord.y-n;
				tmp.suiv = NULL;
				tmp.multiple = NULL;
				ajoute_element(l,tmp);
			}
			if (dpardn(dm,coord,n))
			{
				*prise = 1;
				tmp.depart.x = coord.x;
				tmp.depart.y = coord.y;
				tmp.arrivee.x = coord.x+n;
				tmp.arrivee.y = coord.y-n;
				tmp.suiv = NULL;
				tmp.multiple = NULL;
				ajoute_element(l,tmp);
			}
		}

		/* Déplacements */
		if (!(*prise))
		{
			for (n = 1; n<tailleDamier; n++)
			{
				if (ddavgn(dm,coord,n))
				{
					tmp.depart.x = coord.x;
					tmp.depart.y = coord.y;
					tmp.arrivee.x = coord.x-n;
					tmp.arrivee.y = coord.y+n;
					tmp.suiv = NULL;
					tmp.multiple = NULL;
					ajoute_element(l,tmp);
				}
				if (ddavdn(dm,coord,n))
				{
					tmp.depart.x = coord.x;
					tmp.depart.y = coord.y;
					tmp.arrivee.x = coord.x+n;
					tmp.arrivee.y = coord.y+n;
					tmp.suiv = NULL;
					tmp.multiple = NULL;
					ajoute_element(l,tmp);
				}
				if (ddargn(dm,coord,n))
				{
					tmp.depart.x = coord.x;
					tmp.depart.y = coord.y;
					tmp.arrivee.x = coord.x-n;
					tmp.arrivee.y = coord.y-n;
					tmp.suiv = NULL;
					tmp.multiple = NULL;
					ajoute_element(l,tmp);
				}
				if (ddardn(dm,coord,n))
				{
					tmp.depart.x = coord.x;
					tmp.depart.y = coord.y;
					tmp.arrivee.x = coord.x+n;
					tmp.arrivee.y = coord.y-n;
					tmp.suiv = NULL;
					tmp.multiple = NULL;
					ajoute_element(l,tmp);
				}
			}
		}
	}
}



/* Remplit la liste des coups possibles pour les coups simples
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier, la liste chainée, les coordonnés de la dame, si le paramètre prise vaut 1, seules les prises seront considérées
 * Sortie: (paramètre) construit la liste associée à *l et qui contient tous les coups qui sont possibles, prise et déplacement simples
 */
void coups_possibles_simples(Tdamier dm, Tcouleur coul, Tliste *l)
{
	Tcoup tmp;
	int x, y, n;
	Tcoord coord;
	int prise = 0;

	/* Prises */
	for (x = 0; x<tailleDamier; x++)
	{
		for (y = 0; y<tailleDamier; y++)
		{
			coord.x = x;
			coord.y = y;
			if (dm[coord.x][coord.y].coul == coul)
			{
				if (dm[coord.x][coord.y].piece == 'P')
				{
					if (ppavg(dm,coord))
					{
						prise = 1;
						tmp.depart.x = coord.x;
						tmp.depart.y = coord.y;
						tmp.arrivee.x = coord.x-2;
						tmp.arrivee.y = coord.y+2;
						tmp.suiv = NULL;
						tmp.multiple = NULL;
						ajoute_element(l,tmp);
					}
					if (ppavd(dm,coord))
					{
						prise = 1;
						tmp.depart.x = coord.x;
						tmp.depart.y = coord.y;
						tmp.arrivee.x = coord.x+2;
						tmp.arrivee.y = coord.y+2;
						tmp.suiv = NULL;
						tmp.multiple = NULL;
						ajoute_element(l,tmp);
					}
					if (pparg(dm,coord))
					{
						prise = 1;
						tmp.depart.x = coord.x;
						tmp.depart.y = coord.y;
						tmp.arrivee.x = coord.x-2;
						tmp.arrivee.y = coord.y-2;
						tmp.suiv = NULL;
						tmp.multiple = NULL;
						ajoute_element(l,tmp);
					}
					if (ppard(dm,coord))
					{
						prise = 1;
						tmp.depart.x = coord.x;
						tmp.depart.y = coord.y;
						tmp.arrivee.x = coord.x+2;
						tmp.arrivee.y = coord.y-2;
						tmp.suiv = NULL;
						tmp.multiple = NULL;
						ajoute_element(l,tmp);
					}
				}
				if (dm[coord.x][coord.y].piece == 'D')
				{
					for (n = 1; n<tailleDamier; n++)
					{
						if (dpavgn(dm,coord,n))
						{
							prise = 1;
							tmp.depart.x = coord.x;
							tmp.depart.y = coord.y;
							tmp.arrivee.x = coord.x-n;
							tmp.arrivee.y = coord.y+n;
							tmp.suiv = NULL;
							tmp.multiple = NULL;
							ajoute_element(l,tmp);
						}
						if (dpavdn(dm,coord,n))
						{
							prise = 1;
							tmp.depart.x = coord.x;
							tmp.depart.y = coord.y;
							tmp.arrivee.x = coord.x+n;
							tmp.arrivee.y = coord.y+n;
							tmp.suiv = NULL;
							tmp.multiple = NULL;
							ajoute_element(l,tmp);
						}
						if (dpargn(dm,coord,n))
						{
							prise = 1;
							tmp.depart.x = coord.x;
							tmp.depart.y = coord.y;
							tmp.arrivee.x = coord.x-n;
							tmp.arrivee.y = coord.y-n;
							tmp.suiv = NULL;
							tmp.multiple = NULL;
							ajoute_element(l,tmp);
						}
						if (dpardn(dm,coord,n))
						{
							prise = 1;
							tmp.depart.x = coord.x;
							tmp.depart.y = coord.y;
							tmp.arrivee.x = coord.x+n;
							tmp.arrivee.y = coord.y-n;
							tmp.suiv = NULL;
							tmp.multiple = NULL;
							ajoute_element(l,tmp);
						}
					}
				}
			}
		}
	}


	/* Déplacements */
	if (!prise)
	{
		for (x = 0; x<tailleDamier; x++)
		{
			for (y = 0; y<tailleDamier; y++)
			{
				coord.x = x;
				coord.y = y;
				if (dm[coord.x][coord.y].coul == coul)
				{
					if (dm[coord.x][coord.y].piece == 'P')
					{
						if (pdavg(dm,coord))
						{
							tmp.depart.x = coord.x;
							tmp.depart.y = coord.y;
							tmp.arrivee.x = coord.x-1;
							tmp.arrivee.y = coord.y+1;
							tmp.suiv = NULL;
							tmp.multiple = NULL;
							ajoute_element(l,tmp);
						}
						if (pdavd(dm,coord))
						{
							tmp.depart.x = coord.x;
							tmp.depart.y = coord.y;
							tmp.arrivee.x = coord.x+1;
							tmp.arrivee.y = coord.y+1;
							tmp.suiv = NULL;
							tmp.multiple = NULL;
							ajoute_element(l,tmp);
						}
						if (pdarg(dm,coord))
						{
							tmp.depart.x = coord.x;
							tmp.depart.y = coord.y;
							tmp.arrivee.x = coord.x-1;
							tmp.arrivee.y = coord.y-1;
							tmp.suiv = NULL;
							tmp.multiple = NULL;
							ajoute_element(l,tmp);
						}
						if (pdard(dm,coord))
						{
							tmp.depart.x = coord.x;
							tmp.depart.y = coord.y;
							tmp.arrivee.x = coord.x+1;
							tmp.arrivee.y = coord.y-1;
							tmp.suiv = NULL;
							tmp.multiple = NULL;
							ajoute_element(l,tmp);
						}
					}
					if (dm[coord.x][coord.y].piece == 'D')
					{
						for (n = 1; n<tailleDamier; n++)
						{
							if (ddavgn(dm,coord,n))
							{
								tmp.depart.x = coord.x;
								tmp.depart.y = coord.y;
								tmp.arrivee.x = coord.x-n;
								tmp.arrivee.y = coord.y+n;
								tmp.suiv = NULL;
								tmp.multiple = NULL;
								ajoute_element(l,tmp);
							}
							if (ddavdn(dm,coord,n))
							{
								tmp.depart.x = coord.x;
								tmp.depart.y = coord.y;
								tmp.arrivee.x = coord.x+n;
								tmp.arrivee.y = coord.y+n;
								tmp.suiv = NULL;
								tmp.multiple = NULL;
								ajoute_element(l,tmp);
							}
							if (ddargn(dm,coord,n))
							{
								tmp.depart.x = coord.x;
								tmp.depart.y = coord.y;
								tmp.arrivee.x = coord.x-n;
								tmp.arrivee.y = coord.y-n;
								tmp.suiv = NULL;
								tmp.multiple = NULL;
								ajoute_element(l,tmp);
							}
							if (ddardn(dm,coord,n))
							{
								tmp.depart.x = coord.x;
								tmp.depart.y = coord.y;
								tmp.arrivee.x = coord.x+n;
								tmp.arrivee.y = coord.y-n;
								tmp.suiv = NULL;
								tmp.multiple = NULL;
								ajoute_element(l,tmp);
							}
						}
					}
				}
			}
		}
	}
}


/* Calcul et chainage des prises multiples
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier, le maillon à chainer
 * Sortie: (en paramètre) Les prises multiples chainées sur le maillon
 */
void prises_multiples(Tdamier dm, Tcoup* maillon)
{
	Tdamier dm2;
	Tliste tmp = NULL;
	int prise = 1;
	int x, y;

	/* Copie du damier (oui c'est moche, lourd, mais c'est facile) */
	for (x=0; x<tailleDamier; x++)
	{
		for (y=0; y<tailleDamier; y++)
		{
			dm2[x][y].coul = dm[x][y].coul;
			dm2[x][y].piece = dm[x][y].piece;
		}
	}

	execute_coup(dm2, *maillon);

	if (dm[maillon->depart.x][maillon->depart.y].piece == 'P')
		coups_possibles_simples_pion(dm2, maillon->arrivee, &tmp, &prise);
	if (dm[maillon->depart.x][maillon->depart.y].piece == 'D')
		coups_possibles_simples_dame(dm2, maillon->arrivee, &tmp, &prise);

	maillon->multiple = tmp;

	while (tmp!=NULL)
	{
		prises_multiples(dm2, tmp);
		tmp = tmp->suiv;
	}
}


/* Sauvegarde des coups possibles pour un camp
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier, et la couleur du camp et la liste chainée
 * Sortie: (paramètre) construit la liste associée à *l et qui contient tous les coups qui sont possibles, prise et déplacement pour le camp considéré
 */
void generateur_coups(Tdamier dm, Tcouleur coul, Tliste *l)
{
	Tcoup tmp;
	int x, y, n;
	Tcoord coord;
	int prise = 0;

	init_liste(l);

	/* Prises */
	for (x = 0; x<tailleDamier; x++)
	{
		for (y = 0; y<tailleDamier; y++)
		{
			coord.x = x;
			coord.y = y;
			if (dm[coord.x][coord.y].coul == coul)
			{
				if (dm[coord.x][coord.y].piece == 'P')
				{
					if (ppavg(dm,coord))
					{
						prise = 1;
						tmp.depart.x = coord.x;
						tmp.depart.y = coord.y;
						tmp.arrivee.x = coord.x-2;
						tmp.arrivee.y = coord.y+2;
						tmp.suiv = NULL;
						tmp.multiple = NULL;
						prises_multiples(dm, &tmp);
						ajoute_element(l, tmp);
					}
					if (ppavd(dm,coord))
					{
						prise = 1;
						tmp.depart.x = coord.x;
						tmp.depart.y = coord.y;
						tmp.arrivee.x = coord.x+2;
						tmp.arrivee.y = coord.y+2;
						tmp.suiv = NULL;
						tmp.multiple = NULL;
						prises_multiples(dm, &tmp);
						ajoute_element(l, tmp);
					}
					if (pparg(dm,coord))
					{
						prise = 1;
						tmp.depart.x = coord.x;
						tmp.depart.y = coord.y;
						tmp.arrivee.x = coord.x-2;
						tmp.arrivee.y = coord.y-2;
						tmp.suiv = NULL;
						tmp.multiple = NULL;
						prises_multiples(dm, &tmp);
						ajoute_element(l, tmp);
					}
					if (ppard(dm,coord))
					{
						prise = 1;
						tmp.depart.x = coord.x;
						tmp.depart.y = coord.y;
						tmp.arrivee.x = coord.x+2;
						tmp.arrivee.y = coord.y-2;
						tmp.suiv = NULL;
						tmp.multiple = NULL;
						prises_multiples(dm, &tmp);
						ajoute_element(l, tmp);
					}
				}
				if (dm[coord.x][coord.y].piece == 'D')
				{
					for (n = 1; n<tailleDamier; n++)
					{
						if (dpavgn(dm,coord,n))
						{
							prise = 1;
							tmp.depart.x = coord.x;
							tmp.depart.y = coord.y;
							tmp.arrivee.x = coord.x-n;
							tmp.arrivee.y = coord.y+n;
							tmp.suiv = NULL;
							tmp.multiple = NULL;
							prises_multiples(dm, &tmp);
							ajoute_element(l, tmp);
						}
						if (dpavdn(dm,coord,n))
						{
							prise = 1;
							tmp.depart.x = coord.x;
							tmp.depart.y = coord.y;
							tmp.arrivee.x = coord.x+n;
							tmp.arrivee.y = coord.y+n;
							tmp.suiv = NULL;
							tmp.multiple = NULL;
							prises_multiples(dm, &tmp);
							ajoute_element(l, tmp);
						}
						if (dpargn(dm,coord,n))
						{
							prise = 1;
							tmp.depart.x = coord.x;
							tmp.depart.y = coord.y;
							tmp.arrivee.x = coord.x-n;
							tmp.arrivee.y = coord.y-n;
							tmp.suiv = NULL;
							tmp.multiple = NULL;
							prises_multiples(dm, &tmp);
							ajoute_element(l, tmp);
						}
						if (dpardn(dm,coord,n))
						{
							prise = 1;
							tmp.depart.x = coord.x;
							tmp.depart.y = coord.y;
							tmp.arrivee.x = coord.x+n;
							tmp.arrivee.y = coord.y-n;
							tmp.suiv = NULL;
							tmp.multiple = NULL;
							prises_multiples(dm, &tmp);
							ajoute_element(l, tmp);
						}
					}
				}
			}
		}
	}
	if (prise)
	{
		int prmax = 0;
		Tliste trace = NULL;
		Tliste lres = NULL;

		linearisation(*l, &trace, 1, &prmax, &lres);
		free_trace(&trace);

		/*
		{
			Tliste tmp2 = *l;
			Tliste tmp = NULL;
			printf("debut !\n");
			while (tmp2!=NULL)
			{
				tmp = tmp2;
				while (tmp!=NULL)
				{
					printf("DEBUG Depart %d %d ; Arrivee %d %d -- ",tmp->depart.x,tmp->depart.y,tmp->arrivee.x,tmp->arrivee.y);
					tmp = tmp->multiple;
					if (tmp != NULL && tmp->suiv != NULL)
						printf ("suiv");
				}
				printf("\n");
				tmp2 = tmp2->suiv;
			}
			printf("fin !\n");
		}
		*/
		nettoyage_multiples(&lres, prmax);

		free_liste(l);
		*l = lres;
	}


	/* Déplacements */
	if (!prise)
	{
		for (x = 0; x<tailleDamier; x++)
		{
			for (y = 0; y<tailleDamier; y++)
			{
				coord.x = x;
				coord.y = y;
				if (dm[coord.x][coord.y].coul == coul)
				{
					if (dm[coord.x][coord.y].piece == 'P')
					{
						if (pdavg(dm,coord))
						{
							tmp.depart.x = coord.x;
							tmp.depart.y = coord.y;
							tmp.arrivee.x = coord.x-1;
							tmp.arrivee.y = coord.y+1;
							tmp.suiv = NULL;
							tmp.multiple = NULL;
							ajoute_element(l,tmp);
						}
						if (pdavd(dm,coord))
						{
							tmp.depart.x = coord.x;
							tmp.depart.y = coord.y;
							tmp.arrivee.x = coord.x+1;
							tmp.arrivee.y = coord.y+1;
							tmp.suiv = NULL;
							tmp.multiple = NULL;
							ajoute_element(l,tmp);
						}
						if (pdarg(dm,coord))
						{
							tmp.depart.x = coord.x;
							tmp.depart.y = coord.y;
							tmp.arrivee.x = coord.x-1;
							tmp.arrivee.y = coord.y-1;
							tmp.suiv = NULL;
							tmp.multiple = NULL;
							ajoute_element(l,tmp);
						}
						if (pdard(dm,coord))
						{
							tmp.depart.x = coord.x;
							tmp.depart.y = coord.y;
							tmp.arrivee.x = coord.x+1;
							tmp.arrivee.y = coord.y-1;
							tmp.suiv = NULL;
							tmp.multiple = NULL;
							ajoute_element(l,tmp);
						}
					}
					if (dm[coord.x][coord.y].piece == 'D')
					{
						for (n = 1; n<tailleDamier; n++)
						{
							if (ddavgn(dm,coord,n))
							{
								tmp.depart.x = coord.x;
								tmp.depart.y = coord.y;
								tmp.arrivee.x = coord.x-n;
								tmp.arrivee.y = coord.y+n;
								tmp.suiv = NULL;
								tmp.multiple = NULL;
								ajoute_element(l,tmp);
							}
							if (ddavdn(dm,coord,n))
							{
								tmp.depart.x = coord.x;
								tmp.depart.y = coord.y;
								tmp.arrivee.x = coord.x+n;
								tmp.arrivee.y = coord.y+n;
								tmp.suiv = NULL;
								tmp.multiple = NULL;
								ajoute_element(l,tmp);
							}
							if (ddargn(dm,coord,n))
							{
								tmp.depart.x = coord.x;
								tmp.depart.y = coord.y;
								tmp.arrivee.x = coord.x-n;
								tmp.arrivee.y = coord.y-n;
								tmp.suiv = NULL;
								tmp.multiple = NULL;
								ajoute_element(l,tmp);
							}
							if (ddardn(dm,coord,n))
							{
								tmp.depart.x = coord.x;
								tmp.depart.y = coord.y;
								tmp.arrivee.x = coord.x+n;
								tmp.arrivee.y = coord.y-n;
								tmp.suiv = NULL;
								tmp.multiple = NULL;
								ajoute_element(l,tmp);
							}
						}
					}
				}
			}
		}
	}
}

/* Détermine si le joueur ne peut plus jouer
 * Auteur: Mathieu Pellegrin
 * Entrée: Le coup, et la liste de coups
 * Sortie: Renvoie 1 si le coup cp figure dans la liste l (0 sinon)
 */
int jeu_bloque(Tdamier dm, Tcouleur camp)
{
	int resultat;
	Tliste l;
	init_liste(&l);
	generateur_coups(dm, camp, &l);
	resultat = (l == NULL ? 1 : 0);
	free_liste(&l);
	return resultat;
}


/* Vérification de la validité d'un coup
 * Auteur: Mathieu Pellegrin
 * Entrée: Le coup, et la liste de coups
 * Sortie: renvoie 1 si le coup cp figure dans la liste l (0 sinon)
 */
int verification_coup(Tcoup cp, Tliste l)
{
	int ok = 0;
	Tliste tmpcoup = &cp;
	Tliste tmplistesuiv = l;
	Tliste tmplistemult = l;

	while (tmplistesuiv!=NULL && !ok)
	{
		tmpcoup = &cp;
		if (compare_elements(tmpcoup, tmplistesuiv))
		{
			tmpcoup = tmpcoup->multiple;
			tmplistemult = tmplistesuiv->multiple;
			while (tmpcoup!=NULL && tmplistemult!=NULL && compare_elements(tmpcoup, tmplistemult))
			{
				tmpcoup = tmpcoup->multiple;
				tmplistemult = tmplistemult->multiple;
			}

			if (tmpcoup==NULL && tmplistemult==NULL)
				ok = 1;
		}
		tmplistesuiv = tmplistesuiv->suiv;
	}
	return ok;
}


/* Affichage des coups possibles pour un camp
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier, et la couleur du camp et la liste chainée
 * Sortie: (écran) affichage des coups contenus dans la liste pointée par l
 */
void affiche_coups_possibles(Tliste l)
{
	affiche_liste(l);
}
