/**
 *      damier.c
 *      Contient les fonctions de la gestion du damier
 *
 *      Mathieu Pellegrin
 *
 *      2009
 *
 **/

/* Inclusions standard */
#include <stdlib.h>
#include <stdio.h>

/* Inclusions du programme */
#include "../typedef.h"
#include "../common.h"
#include "damier.h"

/* Initialisation du damier
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier
 * Sortie: (paramètres) le damier "vidé"
 */
void init_damier(Tdamier dm)
{
	int x,y;
	for (x=0; x<tailleDamier; x++)
	{
		for (y=0; y<tailleDamier; y++)
		{
			dm[x][y].coul = 'V';	/* 'V' = vide  */
			dm[x][y].piece = 'P';	/* 'P' = piece */
		}
	}
	for (x=0; x<tailleDamier; x++)
	{
		for (y=0; y<tailleDamier/2-1; y++)
		{
			if ((x+y)%2==0)
				dm[x][y].coul = 'B';
			if ((x+y+1)%2==0)
				dm[x][tailleDamier-y-1].coul = 'N';
		}
	}

/*
	dm[1][1].piece = 'P';
	dm[1][1].coul  = 'N';
	dm[3][3].piece = 'P';
	dm[3][3].coul  = 'N';
	dm[5][5].piece = 'P';
	dm[5][5].coul  = 'N';
	dm[7][7].piece = 'P';
	dm[7][7].coul  = 'N';
	dm[5][7].piece = 'P';
	dm[5][7].coul  = 'N';
	dm[3][7].piece = 'P';
	dm[3][7].coul  = 'N';
	dm[4][4].piece = 'P';
	dm[4][4].coul  = 'B';
*/

/*
	dm[2][3].piece = 'D';
	dm[4][3].piece = 'D';
*/
}


/* Exécute un coup en mettant à jour la structure du damier
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier, le coup à mettre à jour
 * Sortie: Rien
 */
void execute_coup(Tdamier dm, Tcoup cp)
{
	int dx, dy;
	int x, y;
	Tcoup* tmp = &cp;
	Tcoup* tmpprec = &cp;

	while (tmp != NULL)
	{
		dx = tmp->arrivee.x-tmp->depart.x;
		dy = tmp->arrivee.y-tmp->depart.y;
		x = tmp->depart.x;
		y = tmp->depart.y;

		if (dx==0 || dy==0)
			break; /* "Oups" */
		dx = (dx>0 ? 1 : -1);
		dy = (dy>0 ? 1 : -1);

		#if DEBUG_LVL>=2
		printf("EXECUTE %d %d %d %d\n",tmp->depart.x,tmp->depart.y,tmp->arrivee.x,tmp->arrivee.y);
		#endif

		dm[tmp->arrivee.x][tmp->arrivee.y].coul = dm[tmp->depart.x][tmp->depart.y].coul;
		dm[tmp->arrivee.x][tmp->arrivee.y].piece = dm[tmp->depart.x][tmp->depart.y].piece;

		while (x!=tmp->arrivee.x && y!=tmp->arrivee.y)
		{
			dm[x][y].coul = 'V';
			x += dx;
			y += dy;
		}

		tmpprec = tmp;
		tmp = tmp->multiple;
	}


	if ((tmpprec->arrivee.y == 0 && dm[tmpprec->arrivee.x][tmpprec->arrivee.y].piece=='P' && dm[tmpprec->arrivee.x][tmpprec->arrivee.y].coul=='N')
	|| (tmpprec->arrivee.y == tailleDamier-1 && dm[tmpprec->arrivee.x][tmpprec->arrivee.y].piece=='P' && dm[tmpprec->arrivee.x][tmpprec->arrivee.y].coul=='B')
	)
		dm[tmpprec->arrivee.x][tmpprec->arrivee.y].piece = 'D';
}

/* Lecture de la liste de coup sans pause ("jeu de la partie")
 * Auteur: Mathieu Pellegrin
 * Entrée: La liste de coups, le damier
 * Sortie: [Ecran]
 */
void lit_sauvegarde_unlock(Tliste liste, Tdamier damier)
{
	Tcoup coup;
	init_damier(damier);
	/*affiche_damier(damier);*/

	while(liste!=NULL)
	{
		coup.depart.x = (liste->depart).x;
		coup.depart.y = (liste->depart).y;
		coup.arrivee.x = (liste->arrivee).x;
		coup.arrivee.y = (liste->arrivee).y;
		coup.multiple = liste->multiple;
		execute_coup(damier, coup);
		liste = liste->suiv;
	}
}


/* Vérifie si les coordonnés sont dans les limites du damier
 * Auteur: Mathieu Pellegrin
 * Entrée: Les coordonnés
 * Sortie: 1 si les coordonnés sont dans les limites du damier et 0 sinon
 */
int limites_damier(Tcoord c)
{
	return ((c.x>=0) && (c.y>=0) && (c.x<tailleDamier) && (c.y<tailleDamier));
}
