/**
 *      pieces.c
 *      Contient les fonctions pour la gestion des pièces
 *
 *      Mathieu Pellegrin
 *
 *      2009
 *
 **/

/* Inclusions standard */
#include <stdlib.h>
#include <stdio.h>

/* Inclusions du programme */
#include "../typedef.h"
#include "../common.h"
#include "pieces.h"

/* Test si les couleurs sont opposées
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier et les coordonnées des pièces
 * Sortie: 1 si les couleurs sont opposées et 0 sinon
 */
int coul_oppos(Tdamier dm, Tcoord p1, Tcoord p2)
{
	if (dm[p1.x][p1.y].coul==adversaire(dm[p2.x][p2.y].coul))
		return 1;
	else
		return 0;
}


/* Inverse la couleur en entrée
 * Auteur: Mathieu Pellegrin
 * Entrée: La couleur à inverser
 * Sortie: En paramètre: 'N' si 'B' et 'B' si 'N'
 */
void inverser_couleur(Tcouleur *coul)
{
	if(*coul=='N')
		*coul = 'B';
	else if(*coul=='B')
		*coul = 'N';
}


/* Retourne la couleur de l'adversaire
 * Auteur: Mathieu Pellegrin
 * Entrée: Une couleur
 * Sortie: La couleur inversée
 */
Tcouleur adversaire(Tcouleur coul)
{
	inverser_couleur(&coul);
	return coul;
}


/* Déplacement des pions en avant gauche
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier et la coordonné de la pièce
 * Sortie: 1 si déplacement possible et 0 si pas de déplacement possible
 */
int pdavg(Tdamier dm, Tcoord p)
{
	/* Les blancs vont en AVANT*/
	if (dm[p.x][p.y].coul=='B')
	{
		if (p.x-1>=0 && p.y+1<tailleDamier && dm[p.x][p.y].coul!='V' && dm[p.x-1][p.y+1].coul=='V')
			return 1;
		else
			return 0;
	}
	else
		return 0;
}


/* Déplacement des pions en avant droit
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier et la coordonné de la pièce
 * Sortie: 1 si déplacement possible et 0 si pas de déplacement possible
 */
int pdavd(Tdamier dm, Tcoord p)
{
	/* Les blancs vont en AVANT*/
	if (dm[p.x][p.y].coul=='B')
	{
		if (p.x+1<tailleDamier && p.y+1<tailleDamier && dm[p.x][p.y].coul!='V' && dm[p.x+1][p.y+1].coul=='V')
			return 1;
		else
			return 0;
	}
	else
		return 0;
}


/* Déplacement des pions en arrière gauche
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier et la coordonné de la pièce
 * Sortie: 1 si déplacement possible et 0 si pas de déplacement possible
 */
int pdarg(Tdamier dm, Tcoord p)
{
	/* Les noirs vont en ARRIERE */
	if (dm[p.x][p.y].coul=='N')
	{
		if (p.x-1>=0 && p.y-1>=0 && dm[p.x][p.y].coul!='V' && dm[p.x-1][p.y-1].coul=='V')
			return 1;
		else
			return 0;
	}
	else
		return 0;
}


/* Déplacement des pions en arrière droit
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier et la coordonné de la pièce
 * Sortie: 1 si déplacement possible et 0 si pas de déplacement possible
 */
int pdard(Tdamier dm, Tcoord p)
{
	/* Les noirs vont en ARRIERE */
	if (dm[p.x][p.y].coul=='N')
	{
		if (p.x+1<tailleDamier && p.y-1>=0 && dm[p.x][p.y].coul!='V' && dm[p.x+1][p.y-1].coul=='V')
			return 1;
		else
			return 0;
	}
	else
		return 0;
}


/* Synthèse de déplacement des pions
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier et la coordonné de la pièce
 * Sortie: 1 si déplacement possible et 0 si pas de déplacement possible
 */
int pdep(Tdamier dm, Tcoord p)
{
	return pdavg(dm,p) || pdavd(dm,p) || pdarg(dm,p) || pdarg(dm,p);
}


/* Prise des pions en avant gauche
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier et la coordonné de la pièce
 * Sortie: 1 si prise possible et 0 si pas de prise possible
 */
int ppavg(Tdamier dm, Tcoord p)
{
	if ( p.x-2>=0 && p.y+2<tailleDamier
	     && dm[p.x][p.y].coul!='V' && dm[p.x-1][p.y+1].coul!='V' && dm[p.x-2][p.y+2].coul=='V'
	     && dm[p.x-1][p.y+1].coul==adversaire(dm[p.x][p.y].coul)
	   )
		return 1;
	else
		return 0;
}


/* Prise des pions en avant droit
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier et la coordonné de la pièce
 * Sortie: 1 si prise possible et 0 si pas de prise possible
 */
int ppavd(Tdamier dm, Tcoord p)
{
	if ( p.x+2<tailleDamier && p.y+2<tailleDamier
	     && dm[p.x][p.y].coul!='V'  && dm[p.x+1][p.y+1].coul!='V' && dm[p.x+2][p.y+2].coul=='V'
	     && dm[p.x+1][p.y+1].coul==adversaire(dm[p.x][p.y].coul)
	   )
		return 1;
	else
		return 0;
}


/* Prise des pions en arrière gauche
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier et la coordonné de la pièce
 * Sortie: 1 si prise possible et 0 si pas de prise possible
 */
int pparg(Tdamier dm, Tcoord p)
{
		if ( p.x-2>=0 && p.y-2>=0
		     && dm[p.x][p.y].coul!='V' && dm[p.x-1][p.y-1].coul!='V' && dm[p.x-2][p.y-2].coul=='V'
		     && dm[p.x-1][p.y-1].coul==adversaire(dm[p.x][p.y].coul)
		   )
			return 1;
		else
			return 0;
}


/* Prise des pions en arrière droit
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier et la coordonné de la pièce
 * Sortie: 1 si prise possible et 0 si pas de prise possible
 */
int ppard(Tdamier dm, Tcoord p)
{
	if (p.x+2<tailleDamier && p.y-2>=0
	     && dm[p.x][p.y].coul!='V' && dm[p.x+1][p.y-1].coul!='V' && dm[p.x+2][p.y-2].coul=='V'
	     && dm[p.x+1][p.y-1].coul==adversaire(dm[p.x][p.y].coul)
	    )
		return 1;
	else
		return 0;
}


/* Synthèse de prise des pions
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier et la coordonné de la pièce
 * Sortie: 1 si prise possible et 0 si pas de prise possible
 */
int ppep(Tdamier dm, Tcoord p)
{
	return ppavg(dm,p) || ppavd(dm,p) || pparg(dm,p) || pparg(dm,p);
}


/* Déplacement des dames en avant gauche
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier, la coordonné de la pièce, et le nombre de cases du déplacement
 * Sortie: 1 si déplacement possible et 0 si pas de déplacement possible
 */
int ddavgn(Tdamier dm, Tcoord p, int n)
{
	int i = 1;

	if (dm[p.x][p.y].coul!='V' && p.x-n>=0 && p.y+n<tailleDamier && dm[p.x-n][p.y+n].coul=='V')
	{
		while (i<=n && dm[p.x-i][p.y+i].coul=='V')
			i++;

		if (i<=n)
			return 0;
		else
			return 1;
	}
	else
		return 0;
}


/* Déplacement des dames en avant droit
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier, la coordonné de la pièce, et le nombre de cases du déplacement
 * Sortie: 1 si déplacement possible et 0 si pas de déplacement possible
 */
int ddavdn(Tdamier dm, Tcoord p, int n)
{
	int i = 1;

	if (dm[p.x][p.y].coul!='V' && p.x+n<tailleDamier && p.y+n<tailleDamier && dm[p.x+n][p.y+n].coul=='V')
	{
		while (i<=n && dm[p.x+i][p.y+i].coul=='V')
			i++;

		if (i<=n)
			return 0;
		else
			return 1;
	}
	else
		return 0;
}


/* Déplacement des dames en arriere gauche
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier, la coordonné de la pièce, et le nombre de cases du déplacement
 * Sortie: 1 si déplacement possible et 0 si pas de déplacement possible
 */
int ddargn(Tdamier dm, Tcoord p, int n)
{
	int i = 1;

	if (dm[p.x][p.y].coul!='V' && p.x-n>=0 && p.y-n>=0 && dm[p.x-n][p.y-n].coul=='V')
	{
		while (i<=n && dm[p.x-i][p.y-i].coul=='V')
			i++;

		if (i<=n)
			return 0;
		else
			return 1;
	}
	else
		return 0;
}


/* Déplacement des dames en arriere droit
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier, la coordonné de la pièce, et le nombre de cases du déplacement
 * Sortie: 1 si déplacement possible et 0 si pas de déplacement possible
 */
int ddardn(Tdamier dm, Tcoord p, int n)
{
	int i = 1;

	if (dm[p.x][p.y].coul!='V' && p.x+n<tailleDamier && p.y-n>=0 && dm[p.x+n][p.y-n].coul=='V')
	{
		while (i<=n && dm[p.x+i][p.y-i].coul=='V')
			i++;

		if (i<=n)
			return 0;
		else
			return 1;
	}
	else
		return 0;
}


/* Synthèse déplacement des dames
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier et la coordonné de la pièce
 * Sortie: 1 si déplacement possible et 0 si pas de déplacement possible
 */
int ddep(Tdamier dm, Tcoord p)
{
	int i = 1;

	/* *2 parce que racine de 2 ça existe pas :P */
	while (i<tailleDamier*2 && !(ddavgn(dm,p,i) || ddavdn(dm,p,i) || ddargn(dm,p,i) || ddargn(dm,p,i)))
		i++;

	if (i<tailleDamier*2)
		return 0;
	else
		return 1;
}


/* Prise des dames en avant gauche
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier, la coordonné de la pièce, et le nombre de cases du déplacement
 * Sortie: 1 si prise possible et 0 si pas de prise possible
 */
int dpavgn(Tdamier dm, Tcoord p, int n)
{
	int i = 1;
	int flag = 0;

	if (dm[p.x][p.y].coul!='V' && p.x-n>=0 && p.y+n<tailleDamier && dm[p.x-n][p.y+n].coul=='V')
	{
		while (i<n && flag<2 && (dm[p.x-i][p.y+i].coul=='V' || dm[p.x-i][p.y+i].coul==adversaire(dm[p.x][p.y].coul)))
		{
			if (dm[p.x-i][p.y+i].coul==adversaire(dm[p.x][p.y].coul))
				flag++;
			i++;
		}

		if (i<n || flag<1 || flag>=2)
			return 0;
		else
			return 1;
	}
	else
		return 0;
}


/* Prise des dames en avant droit
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier, la coordonné de la pièce, et le nombre de cases du déplacement
 * Sortie: 1 si prise possible et 0 si pas de prise possible
 */
int dpavdn(Tdamier dm, Tcoord p, int n)
{
	int i = 1;
	int flag = 0;

	if (dm[p.x][p.y].coul!='V' && p.x+n<tailleDamier && p.y+n<tailleDamier && dm[p.x+n][p.y+n].coul=='V')
	{
		while (i<n && flag<2 && (dm[p.x+i][p.y+i].coul=='V' || dm[p.x+i][p.y+i].coul==adversaire(dm[p.x][p.y].coul)))
		{
			if (dm[p.x+i][p.y+i].coul==adversaire(dm[p.x][p.y].coul))
				flag++;
			i++;
		}

		if (i<n || flag<1 || flag>=2)
			return 0;
		else
			return 1;
	}
	else
		return 0;
}


/* Prise des dames en arrière gauche
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier, la coordonné de la pièce, et le nombre de cases du déplacement
 * Sortie: 1 si prise possible et 0 si pas de prise possible
 */
int dpargn(Tdamier dm, Tcoord p, int n)
{
	int i = 1;
	int flag = 0;

	if (dm[p.x][p.y].coul!='V' && p.x-n>=0 && p.y-n>=0 && dm[p.x-n][p.y-n].coul=='V')
	{
		while (i<n && flag<2 && (dm[p.x-i][p.y-i].coul=='V' || dm[p.x-i][p.y-i].coul==adversaire(dm[p.x][p.y].coul)))
		{
			if (dm[p.x-i][p.y-i].coul==adversaire(dm[p.x][p.y].coul))
				flag++;
			i++;
		}

		if (i<n || flag<1 || flag>=2)
			return 0;
		else
			return 1;
	}
	else
		return 0;
}


/* Prise des dames en arrière droit
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier, la coordonné de la pièce, et le nombre de cases du déplacement
 * Sortie: 1 si prise possible et 0 si pas de prise possible
 */
int dpardn(Tdamier dm, Tcoord p, int n)
{
	int i = 1;
	int flag = 0;

	if (dm[p.x][p.y].coul!='V' && p.x+n<tailleDamier && p.y-n>=0 && dm[p.x+n][p.y-n].coul=='V')
	{
		while (i<n && flag<2 && (dm[p.x+i][p.y-i].coul=='V' || dm[p.x+i][p.y-i].coul==adversaire(dm[p.x][p.y].coul)))
		{
			if (dm[p.x+i][p.y-i].coul==adversaire(dm[p.x][p.y].coul))
				flag++;
			i++;
		}

		if (i<n || flag<1 || flag>=2)
			return 0;
		else
			return 1;
	}
	else
		return 0;
}


/* Synthèse de prise des dames
 * Auteur: Mathieu Pellegrin
 * Entrée: Le damier et la coordonné de la pièce
 * Sortie: 1 si déplacement possible et 0 si pas de prise possible
 */
int dpep(Tdamier dm, Tcoord p)
{
	int i = 1;

	/* *2 parce que racine de 2 ça existe pas :P */
	while (i<tailleDamier*2 && !(dpavgn(dm,p,i) || dpavdn(dm,p,i) || dpargn(dm,p,i) || dpargn(dm,p,i)))
		i++;

	if (i<tailleDamier*2)
		return 0;
	else
		return 1;
}






/*printf("%d %d %d %d %d %d %d %c %c\n",p.x,p.y,(p.x+2<tailleDamier && p.y-2>=0),(dm[p.x][p.y].coul!='V'),(dm[p.x+1][p.y-1].coul!='V'),(dm[p.x+2][p.y-2].coul=='V'),(dm[p.x+1][p.y-1].coul==adversaire(dm[p.x][p.y].coul)),dm[p.x][p.y].coul,adversaire(dm[p.x][p.y].coul));*/
