/**
 *      prises_multiples.c
 *      Contient les fonctions pour la gestion des prises multiples
 *
 *      Mathieu Pellegrin
 *
 *      2009
 *
 **/

/* Inclusions standard */
#include <stdlib.h>
#include <stdio.h>

/* Inclusions du programme */
#include "../typedef.h"
#include "../common.h"
#include "gestion_liste.h"
#include "prises_multiples.h"


/* Calcul de la longueur du coup dans la prise multiple
 * Auteur: Mathieu Pellegrin
 * Entrée: Le coup de départ
 * Sortie: La longueur du coup
 */
int longueur_coup(Tcoup cp)
{
	int i = 0;
	Tcoup* tmp = &cp;

	while (tmp != NULL)
	{
		i++;
		tmp = tmp->multiple;
	}

	return i;
}


/* Copie inverse de la liste chainée sur multiple (les suiv des multiples sont NULL)
 * Auteur: Mathieu Pellegrin
 * Entrée: la liste
 * Sortie: la liste copiée inversée
 */
void copie_inverse(Tliste lm, Tliste *l_inv)
{
	Tcoup* tmporig = lm;
	Tcoup* tmpcopie = NULL;

	while (tmporig != NULL)
	{
		copie_maillon(tmporig, &tmpcopie);
		tmpcopie->suiv = NULL;
		tmpcopie->multiple = *l_inv;
		*l_inv = tmpcopie;
		tmporig = tmporig->multiple;
	}
}


/* Linéarisation de la liste des prises multiples
 * Auteur: Mathieu Pellegrin
 * Entrée: La liste, une liste temporaire, la profondeur, la profondeur maximale
 * Sortie: (en paramètre) La liste linéarisée
 */
void linearisation(Tliste l, Tliste *trace, int pr, int *prmax, Tliste *lres)
{
	Tcoup cp;
	cp.depart.x = l->depart.x;
	cp.depart.y = l->depart.y;
	cp.arrivee.x = l->arrivee.x;
	cp.arrivee.y = l->arrivee.y;
	cp.suiv = NULL;
	cp.multiple = NULL;

	ajoute_trace(trace, cp);
	if (l->multiple != NULL)
	{
		linearisation(l->multiple, trace, pr+1, prmax, lres);
	}
	else
	{
		accrocher_trace(*trace, lres);

		if (pr > *prmax)
		{
			*prmax = pr;
		}
	}
	retire_trace(trace);

	if (l->suiv != NULL)
		linearisation(l->suiv, trace, pr, prmax, lres);
}


/* Nettoyage de la liste pour ne conserver que les prises les plus longues (profondeur prmax)
 * Auteur: Mathieu Pellegrin
 * Entrée: La liste, la profondeur maximale
 * Sortie: (en paramètre) La liste nettoyée
 */
void nettoyage_multiples(Tliste *l, int prmax)
{
	Tliste tmp = *l;
	Tliste* prec = l;
	int n = 0;

	while (tmp != NULL)
	{
		n = longueur_coup(*tmp);

		if (n < prmax)
		{
				*prec = tmp->suiv;
				free_maillon(tmp);
				tmp = *prec;
		}
		else
		{
			prec = &(tmp->suiv);
			tmp = tmp->suiv;
		}
	}

}


/* Ajout d'un élément dans la liste trace
 * Auteur: Mathieu Pellegrin
 * Entrée: La liste trace
 * Sortie: (en paramètre) La liste avec son nouvel élément
 */
void ajoute_trace(Tliste* trace, Tcoup cp)
{
	Tcoup* tmp;
	tmp = (Tcoup*)malloc(sizeof(Tcoup));
	(tmp->depart).x = cp.depart.x;
	(tmp->depart).y = cp.depart.y;
	(tmp->arrivee).x = cp.arrivee.x;
	(tmp->arrivee).y = cp.arrivee.y;
	tmp->suiv = NULL;
	tmp->multiple = *trace;
	if (*trace != NULL)
		(*trace)->suiv = tmp;
	*trace = tmp;


	#if DEBUG_LVL>=3
	printf("ALLOC ajoute_trace %p %d %d %d %d %p %p\n",(void*)tmp,tmp->depart.x,tmp->depart.y,tmp->arrivee.x,tmp->arrivee.y,(void*)tmp->multiple,(void*)tmp->suiv);
	#endif
}


/* Retrait d'un élément de la liste trace
 * Auteur: Mathieu Pellegrin
 * Entrée: La liste trace
 * Sortie: (en paramètre) La liste avec son nouvel élément
 */
void retire_trace(Tliste* trace)
{
	Tcoup* tmp;
	tmp = *trace;
	if (tmp->multiple != NULL)
		tmp->multiple->suiv = tmp->suiv;
	if (tmp->suiv != NULL)
		tmp->suiv->multiple = tmp->multiple;
	else
		*trace = tmp->multiple;
	free(tmp);
}


/* Accrochage (+copie) de la liste trace à la liste lres
 * Auteur: Mathieu Pellegrin
 * Entrée: La liste trace
 * Sortie: (en paramètre) La liste avec trace accrochée
 */
void accrocher_trace(Tliste trace, Tliste *lres)
{
	Tliste tmp = NULL;

	copie_inverse(trace,&tmp);
	tmp->suiv = *lres;
	*lres = tmp;
}


/* Suppression totale de la liste trace
 * Auteur: Mathieu Pellegrin
 * Entrée: La liste trace
 * Sortie: Rien
 */
void free_trace(Tliste *trace)
{
	Tliste tmp = *trace;

	while (tmp!=NULL)
	{
		free(tmp);
		tmp = tmp->multiple;
	}
	*trace = NULL;
}
